@@ -2,15 +2,18 @@
 #this repository contains the full copyright notices and license terms.
 {
     'name': 'Project Plan',
+    'name_bg_BG': 'Планиране на проекти',
     'name_de_DE': 'Projekte Planung',
     'name_es_CO': 'Planeación de Proyectos',
     'name_es_ES': 'Planificación de proyecto',
     'name_fr_FR': 'Planification de projet',
-    'version': '1.8.0',
+    'version': '2.0.1',
     'author': 'B2CK',
     'email': 'info@b2ck.com',
     'website': 'http://www.tryton.org/',
     'description': '''Add planning capabilities on projects.
+''',
+    'description_bg_BG': '''Добавя възможност за планиране на проекти
 ''',
     'description_de_DE': '''Projektmodul Planung:
     - Fügt Planungsmöglichkeiten für Projekte hinzu
@@ -31,6 +34,7 @@
         'allocation.xml',
     ],
     'translation': [
+        'bg_BG.csv',
         'de_DE.csv',
         'es_CO.csv',
         'es_ES.csv',
